use bevy::prelude::*;
use bevy_rapier2d::prelude::*;
use player::{add_player, move_player};

fn main() {
    App::new()
        .add_plugins(DefaultPlugins)
        .add_plugins(RapierPhysicsPlugin::<NoUserData>::pixels_per_meter(100.0))
        .add_plugins(RapierDebugRenderPlugin::default())
        .add_systems(Startup, (setup_camera, add_player))
        .add_systems(Update, move_player)
        .run();
}

fn setup_camera(mut commands: Commands) {
    commands.spawn(Camera2dBundle {
        transform: Transform::from_xyz(0., 0., 100.),
        ..default()
    });
}

mod scene {
    use bevy::prelude::*;
    use bevy_rapier2d::prelude::*;

    pub fn setup_scene(mut commands: Commands) {}
}

mod player {
    use bevy::prelude::*;
    use bevy_rapier2d::prelude::*;
    use std::f32::consts::PI;

    const TURN_SPEED: f32 = PI / 100.;
    const FW_SPEED: f32 = 5.;
    const BW_SPEED: f32 = 3.;
    const SIDEWARD_SPEED: f32 = 3.5;

    #[derive(Component)]
    pub struct Player {
        move_cooldown: Timer,
    }

    pub fn add_player(mut commands: Commands, asset_server: Res<AssetServer>) {
        commands
            .spawn((
                Player {
                    move_cooldown: Timer::from_seconds(0.01, TimerMode::Repeating),
                },
                SpriteBundle {
                    texture: asset_server.load("sprites/player.png"),
                    transform: Transform::from_xyz(0., 0., 0.),
                    ..default()
                },
            ))
            .insert(Collider::cuboid(8., 8.));
    }

    pub fn move_player(
        kb_input: Res<ButtonInput<KeyCode>>,
        mut query: Query<(&mut Transform, &mut Player)>,
        time: Res<Time>,
    ) {
        let (mut transform, mut player) = query.single_mut();
        if player.move_cooldown.tick(time.delta()).finished() {
            let mut moved = false;
            let (mut mv_forward, mut mv_sideward, mut rot) = (0., 0., 0.);

            if kb_input.pressed(KeyCode::KeyA) {
                moved = true;
                rot += TURN_SPEED;
            }
            if kb_input.pressed(KeyCode::KeyD) {
                moved = true;
                rot -= TURN_SPEED;
            }
            if kb_input.pressed(KeyCode::KeyW) {
                moved = true;
                mv_forward += FW_SPEED;
            }
            if kb_input.pressed(KeyCode::KeyE) {
                moved = true;
                mv_sideward += SIDEWARD_SPEED;
            }
            if kb_input.pressed(KeyCode::KeyQ) {
                moved = true;
                mv_sideward -= SIDEWARD_SPEED;
            }
            if kb_input.pressed(KeyCode::KeyS) {
                moved = true;
                mv_forward -= BW_SPEED;
            }

            if moved {
                player.move_cooldown.reset();
                transform.rotation *= Quat::from_rotation_z(rot);
                let fw = transform.up();
                let right = transform.right();
                transform.translation += fw * mv_forward + right * mv_sideward;
            }
        }
    }
}
